@echo off 
REM *****************************************************************
REM This DOS program converts one or more rectilinear cube faces to
REM equirectangular output, using PanoTools. 
REM Only compatible with cmd.exe under Win 2000/XP.
REM
REM Create a shortcut pointing to this batch file, and put the shortcut in the 'Send To' folder
REM located at C:\Documents and Settings\%USERNAME%\SendTo
REM Then rightclick on a selected input image(s) you wish to convert, and choose
REM SendTo this batch file. You can also select multiple images simultaneously,
REM and then right click SendTo.
REM
REM Or, just drag an image(s) onto this program's icon
REM
REM Or, just enter from command line: [this program name] [image name]
REM
REM The output image will appear in the same folder as the input image.
REM
REM Note: a strange quirk with PTStitcher. If I want to merge an input cube face with
REM an input equirect image to create an output equi image - this only works for the
REM zenith and nadir. Not for any of the other cube faces. The other 4 faces will be obscured
REM by the equi input image.
REM Also, the zenith/nadir have to be listed BEFORE the equi image in the stitcher script.


REM *******************************************************************************

REM Cube face file names have suffixes _11, _22, _33, _44, _55, _66
REM Change these variables if you need to change the suffix for cube face file names.
REM If you decide to change these variables, make sure that they have 
REM at least 1 character and no more than 4 characters
SET Tile1Suffix=_11
SET Tile2Suffix=_22
SET Tile3Suffix=_33
SET Tile4Suffix=_44
SET Tile5Suffix=_55
SET Tile6Suffix=_66



REM Initialization
SET searchreal=checknum
goto :initialstuff
:returninitialstuff


SET Script=C2PScript.txt

REM Switch determines whether to accept equirectangular input images
REM in addition to the cube faces. To turn off, let EqInput = blank
REM SET EqInput=true
SET EqInput=



REM ********************************************************************************
REM Search all input arguments for control parameters
REM such as width. Doing this
REM allows the user to specify control parameters ANYWHERE
REM in the input line, even last. Also search for an equirectangular
REM input image, which would be merged with the rectilinear input
REM cube faces.

REM We set width = nothing, Very important here.
SET width=

REM Search all input arguments for output pano width
call :getpanowidth %*

REM Eqimage is the equirectangular image file
SET Eqimage=

REM Search all input arguments for equirectangular input image
REM If found, put in Eqimage variable
if not "%EqInput%" == "" call :checkequi %*


REM *******************************************************************************
REM Now %1 points to either a cube face input image or an equirectangular input image.

REM We must check that the input file has name ending with Tile1Suffix, Tile2Suffix, ...
REM or 0000, 0001, 0002, 0003, 0004, 0005.
REM The face order is exactly the same as produced by PanoCube
REM when it produces output tiles for editing.
REM If not, then we can assume it is an equirectangular file, and should appear LAST in
REM the script file (ie. bottom layer). This will allow you to merge a cube face with
REM an already existing equirectangular image.


REM Number of digits to remove from output filename
REM This variable is set if a cube face has been detected.
set digits=



REM tmp is name of input file. Any surrounding quotes are automatically
REM removed using ~1. Since tmp has no quotes in it, we add
REM quotes to it for tmp1. This will avoid any errors in the next 'if' statement.
set tmp=%~n1
set tmp4="%tmp:~-4%"



REM Find the length of the file name suffixes
set suffixLen=
set tmp2=
if "%Tile1Suffix:~0,-4%" == "" (
  set suffixLen=4
  set tmp2="%tmp:~-4%"
)
if "%Tile1Suffix:~0,-3%" == "" (
  set suffixLen=3
  set tmp2="%tmp:~-3%"
)
if "%Tile1Suffix:~0,-2%" == "" (
  set suffixLen=2
  set tmp2="%tmp:~-2%"
)
if "%Tile1Suffix:~0,-1%" == "" (
  set suffixLen=1
  set tmp2="%tmp:~-1%"
)


REM Cube face 1
REM Note: we need to be worried about spaces in file name,
REM which would cause an error in an 'if' statement. Since tmp has no
REM quotes in it, we can add surrounding quotes to the if statement, to avoid
REM error.
if %tmp2% == "%Tile1Suffix%" set digits=%suffixLen%
if %tmp4% == "0000" set digits=4

REM Cube face 2
if %tmp2% == "%Tile2Suffix%" set digits=%suffixLen%
if %tmp4% == "0001" set digits=4

REM Cube face 3
if %tmp2% == "%Tile3Suffix%" set digits=%suffixLen%
if %tmp4% == "0002" set digits=4

REM Cube face 4
if %tmp2%== "%Tile4Suffix%" set digits=%suffixLen%
if %tmp4% == "0003" set digits=4

REM Cube face 5
if %tmp2% == "%Tile5Suffix%" set digits=%suffixLen%
if %tmp4% == "0004" set digits=4

REM Cube face 6
if %tmp2% == "%Tile6Suffix%" set digits=%suffixLen%
if %tmp4% == "0005" set digits=4



REM If digits is not blank, then we have a cube face
if not "%digits%" == "" goto foundcube


REM If we have reached this point here, then we should assume we have an
REM equirectangular input image if EqInput is not blank. Since we already searched all 
REM input arguments for an equi image input, we shift and continue searching for a cube face input. 
REM If EqInput IS blank, then we should assume it is an improperly named image that does
REM not have the correct file name suffix - so give error message and skip that image.
if not "%EqInput%" == "" (
     shift /1
     goto checkargs
)
if "%EqInput%" == "" (
     call :wrongsuffix "%~1"
     shift /1
     goto checkargs
)
shift /1
goto checkargs


:foundcube
REM If we reach this point, then a cube face input image has been found

REM ***********************************************************************
REM Introductory message
REM cls

echo Convert Rectilinear Cube Face(s) to equirectangular image using PanoTools.
echo.
echo Input image size(s) must be Width x Height = N x N [pixels]
echo Output image size will be Width x Height = 2Q x Q [pixels]

echo Interpolation method : %interp%
echo.

REM ************************************************************************
REM Get positive numbers for image width and height.
REM If width already has a value (from commandline), then skip
REM input routine and just calculate height. Recall that we previously set width=blank
REM and then called a subroutine to check the input arguments for a width number.

REM WATCH out for delayed environ. variable expansion!
if not "%width%" == "" goto hhh2   
    call :getnum "Enter desired width (even number) of output image (pixels) : "
    if "%posnum%" == "0" goto :eof
    if "%posnum%" == "" goto :eof 
    REM Had to check for empty posnum before assigning to width, or else set /a width may give error
    set width=
    set /a width=%posnum%
 
:hhh2
REM Now calculate height of the Output Image
set height=
set /a height=%width%/2
 
if "%height%" == "0" goto :eof
if "%height%" == "" goto :eof

REM We now recalculate width from height just to be sure that width is even number (for Equi)
set width=
set /a width=%height%*2


REM ********************************************************
REM Naming of output equirectangular image file

REM Name of input file
set tmp=%~n1

REM We place output Pano in same directory as the input file %1
REM We have to add quotes here because %~1 automatically removes
REM any surrounding quotes from %1.
if "%digits%" == "1" (
     SET Pano="%~dp1%tmp:~0,-1%(w%width%).tif"
)
if "%digits%" == "2" (
     SET Pano="%~dp1%tmp:~0,-2%(w%width%).tif"
)
if "%digits%" == "3" (
     SET Pano="%~dp1%tmp:~0,-3%(w%width%).tif"
)
if "%digits%" == "4" (
     SET Pano="%~dp1%tmp:~0,-4%(w%width%).tif"
)



REM ***********************************************************
REM Create script file used by PTStitcher

REM OUTPUT image is equirectangular with width = 2 x height
REM We must specify u0 - feather width = 0 here when merging cube faces
REM or else we will see black borders in the final pano
echo # OUTPUT Equirectangular image > %Script%
echo # f4=equirectangular output format, v=hfov, w=width, h=height  >> %Script%
echo p w%width% h%height% f4 v360 n"TIFF" u0 >> %Script%
echo. >> %Script%

REM The 'm' line sets the interpolation parameters
REM poly3 is worst, sinc1024 is best - except that sinc causes ringing at high contrast edges
REM i0=poly3, i1=spline16, i2=spline36, i4=spline64, i3=sinc256, i7=sinc1024
echo # The 'm' line sets the interpolation parameters >> %Script%
echo # i0=poly3, i1=spline16, i2=spline36, i4=spline64, i3=sinc256, i7=sinc1024 >> %Script%
echo m g1 %interpolate% >> %Script%
echo. >> %Script% 

REM INPUT rectilinear/equirectangular image
echo # INPUT image specification  >> %Script%
echo # f4=equirectangular input image  >> %Script%
echo # f0=rectilinear input format  >> %Script%
echo # y=yaw angle, p=pitch angle, r=roll angle, v=hfov  >> %Script%



REM Put each cube face into script.
REM :scriptface will add the stuff to script file. Will also give
REM output error message if %1 is not a number, not an interpolation
REM specification, not a valid file type, or not an existing file.
:anotherface
if not "%~1" == "" (
   call :scriptface "%~1"
   shift /1
   goto anotherface
)


REM Put Equirectangular image into script (LAST only, very important to make it LAST in script)
REM if it was given as an input.
if not "%EqInput%" == "" call :scriptequi


REM ******************************************************************
REM Now run PTStitcher to convert cube faces to equirectangular format

echo.

REM Delete if file exists, or else PanoTools PTStitcher gets confused
if exist %Pano% del %Pano%

echo [Converting Cube Faces to Equirectangular %width% x %height% ...]
%PTStitch% -o %Pano% %Script%

REM Should we delete the script file when done?
if exist %Script% del %Script%

goto :eof


REM ******************************************************************************************************
REM SUBROUTINE - Input is the filename that has the wrong suffix
:wrongsuffix
echo.
echo The following file has the wrong suffix and therefore cannot be processed:
echo %1
echo.
echo To convert one or more cube faces to equirectangular pano format, their
echo filenames should end with %Tile1Suffix%, %Tile2Suffix%, %Tile3Suffix%, %Tile4Suffix%, %Tile5Suffix%, %Tile6Suffix%
echo or with 0000, 0001, 0002, 0003, 0004, 0005
echo.
pause
goto :eof

REM Return from subroutine

REM ******************************************************************************************************
REM SUBROUTINE
:scriptface

REM Input %1 is the name of a cube face
REM This routine determines which cube face it is - face 1, 2, 3, 4, 5, or 6
REM and creates the appropriate lines in the ptstitcher script file.
REM Also gives output error messages if bad input arguments.



REM Verify that %1 is actually an image
SET searchreal=checknum
call :searchfirstimage %1
if not "%shiftargs%" == "0" goto :eof



REM tmp is file name only.
REM %~1 removes quotes from %1 if they are present (may or may not be present).
set tmp=%~n1
set tmp4="%tmp:~-4%"


REM Find the length of the file name suffixes
set tmp2=
if "%suffixLen%" == "4" set tmp2="%tmp:~-4%"
if "%suffixLen%" == "3" set tmp2="%tmp:~-3%"
if "%suffixLen%" == "2" set tmp2="%tmp:~-2%"
if "%suffixLen%" == "1" set tmp2="%tmp:~-1%"


REM Get last 3 digits of file name
REM If last digits match the TileSuffix, then create script lines for cube face 1
REM Note: we need to be worried about spaces in file name,
REM which would cause an error in an 'if' statement. Since tmp has no
REM quotes in it, we can add surrounding quotes to the if statement, to avoid
REM error (quotes contained in tmp1)
if %tmp2% == "%Tile1Suffix%" (
   echo i n"%~1" >> %Script%
   echo o f0 y0 p0 r0 v%cubefov% >> %Script%
   echo. >> %Script%
   goto :eof 
)
if %tmp4% == "0000" (
   echo i n"%~1" >> %Script%
   echo o f0 y0 p0 r0 v%cubefov% >> %Script%
   echo. >> %Script%
   goto :eof 
)

REM Get last 3 digits of file name
REM If last digits match the TileSuffix, then create script lines for cube face 2
if %tmp2% == "%Tile2Suffix%" (
   echo i n"%~1" >> %Script%
   echo o f0 y90 p0 r0 v%cubefov% >> %Script%
   echo. >> %Script% 
   goto :eof 
)
if %tmp4% == "0001" (
   echo i n"%~1" >> %Script%
   echo o f0 y90 p0 r0 v%cubefov% >> %Script%
   echo. >> %Script% 
   goto :eof 
)

REM Get last 3 digits of file name
REM If last digits match the TileSuffix, then create script lines for cube face 3
if %tmp2% == "%Tile3Suffix%" (
   echo i n"%~1" >> %Script%
   echo o f0 y180 p0 r0 v%cubefov% >> %Script%
   echo. >> %Script%
   goto :eof 
)
if %tmp4% == "0002" (
   echo i n"%~1" >> %Script%
   echo o f0 y180 p0 r0 v%cubefov%>> %Script%
   echo. >> %Script%
   goto :eof 
)


REM Get last 3 digits of file name
REM If last digits match the TileSuffix, then create script lines for cube face 4
if %tmp2% == "%Tile4Suffix%" (
   echo i n"%~1" >> %Script%
   echo o f0 y-90 p0 r0 v%cubefov% >> %Script%
   echo. >> %Script%
   goto :eof
)
if %tmp4% == "0003" (
   echo i n"%~1" >> %Script%
   echo o f0 y-90 p0 r0 v%cubefov% >> %Script%
   echo. >> %Script%
   goto :eof  
)

REM Get last 3 digits of file name
REM If last digits match the TileSuffix, then create script lines for cube face 5
if %tmp2% == "%Tile5Suffix%" (
   echo i n"%~1" >> %Script%
   echo o f0 y0 p90 r0 v%cubefov% >> %Script%
   echo. >> %Script%
   goto :eof  
)
if %tmp4% == "0004" (
   echo i n"%~1" >> %Script%
   echo o f0 y0 p90 r0 v%cubefov% >> %Script%
   echo. >> %Script%
   goto :eof  
)

REM Get last 3 digits of file name
REM If last digits match the TileSuffix, then create script lines for cube face 6
if %tmp2% == "%Tile6Suffix%" (
   echo i n"%~1" >> %Script%
   echo o f0 y0 p-90 r0 v%cubefov% >> %Script%
   echo. >> %Script%
   goto :eof  
)
if %tmp4% == "0005" (
   echo i n"%~1" >> %Script%
   echo o f0 y0 p-90 r0 v%cubefov% >> %Script%
   echo. >> %Script%
   goto :eof  
)



REM If we have reached this point here, then we should assume we have an
REM equirectangular input image if EqInput is not blank. The equirect. image will be processed 
REM outside this subroutine. If EqInput IS blank, then it is an improperly named file with the
REM wrong suffix.
if not "%EqInput%" == "" (
     goto :eof
)
if "%EqInput%" == "" (
     call :wrongsuffix "%~1"
     goto :eof
)

goto :eof
REM return from subroutine

REM *****************************************************************************************************
SUBROUTINE - Add Equirectangular image to script
:scriptequi

REM Warning - Eqimage will contain quotes
if not a%Eqimage%a == aa (
   echo i n%Eqimage% >> %Script%
   echo o f4 y0 p0 r0 v360 >> %Script%
   echo. >> %Script%
)

goto :eof

REM *************************************************************************************************
REM SUBROUTINE - Search all the input arguments, see if one of them is an
REM equirectangular input image.
:checkequi


REM We must give all inputs to :searchfirstimage explicitly instead of
REM using %* because %* cannot be shifted, but the input args themselves will.
SET searchreal=checknum
call :searchfirstimage %1 %2 %3 %4 %5 %6 %7 %8 %9
:shifterloop2
if not "%shiftargs%" == "0" (
   shift /1
   SET /a shiftargs=shiftargs-1
   goto :shifterloop2
)

REM If %1 argument is blank, then stop because we ran out of inputs to process
if "%~1" == "" goto :eof

REM Valid Image now found

REM Now %1 points to either a cube face or an equirect. input image.

REM We must check that the input file has name ending with %Tile1Suffix%
REM or 0000, 0001, 0002, 0003, 0004, 0005.
REM The naming convention and face order is exactly the same as produced by PanoCube
REM when it produces output tiles for editing.
REM If not, then we can assume it is an equirectangular file, and should appear LAST in
REM the script file (ie. bottom layer). This will allow you to merge a cube face with
REM an already existing equirectangular image.

REM tmp is name of input file. Any surrounding quotes are automatically
REM removed using ~1. Since tmp has no quotes in it, we add
REM quotes to it for tmp1. This will avoid any errors in the next 'if' statement.
set tmp=%~n1
set tmp4="%tmp:~-4%"


REM Find the length of the file name suffixes
set tmp2=
if "%suffixLen%" == "4" set tmp2="%tmp:~-4%"
if "%suffixLen%" == "3" set tmp2="%tmp:~-3%"
if "%suffixLen%" == "2" set tmp2="%tmp:~-2%"
if "%suffixLen%" == "1" set tmp2="%tmp:~-1%"



REM Cube face 1
REM Note: we need to be worried about spaces in file name,
REM which would cause an error in an 'if' statement. Since tmp has no
REM quotes in it, we can add surrounding quotes to the if statement, to avoid
REM error.
if %tmp2% == "%Tile1Suffix%" goto chckequiAA
if %tmp4% == "0000" goto chckequiAA

REM Cube face 2
if %tmp2% == "%Tile2Suffix%" goto chckequiAA
if %tmp4% == "0001" goto chckequiAA

REM Cube face 3
if %tmp2% == "%Tile3Suffix%" goto chckequiAA
if %tmp4% == "0002" goto chckequiAA

REM Cube face 4
if %tmp2%== "%Tile4Suffix%" goto chckequiAA
if %tmp4% == "0003" goto chckequiAA

REM Cube face 5
if %tmp2% == "%Tile5Suffix%" goto chckequiAA
if %tmp4% == "0004" goto chckequiAA

REM Cube face 6
if %tmp2% == "%Tile6Suffix%" goto chckequiAA
if %tmp4% == "0005" goto chckequiAA


REM If we have reached this point here, then we should assume we have an
REM equirectangular image that should go on the bottom layer of the script.
REM We save this equi image so we can add it to the script LAST. 
SET Eqimage= 
SET Eqimage="%~1"
goto :eof


:chckequiAA
shift /1
goto checkequi

REM End of subroutine


REM ******************************************************************************
:instruct
REM Print out version number
goto dfpversion
:returndfpversion

echo This DOS program will convert one or more rectilinear cube face images 
echo [size = N x N with 90 degrees hfov] to an equirectangular format image 
echo [size = 2Q x Q with 360 degrees hfov] using PanoTools. 
echo.
echo No antialiasing filter is used in this conversion.
echo.
echo You can specify 1 or more (up to 6) cube faces as input.
echo The input image file names should end with suffixes %Tile1Suffix%, %Tile2Suffix%, %Tile3Suffix%,
echo %Tile4Suffix%, %Tile5Suffix%, and %Tile6Suffix%.  OR, they should end with suffixes 0000, 0001, 0002,
echo  0003, 0004, and 0005. The suffixes determine the position of each face 
echo on the cube. The face order is exactly the same as that produced by 
echo PanoCube when it creates output faces for editing.
echo.
pause

echo.
echo To run this program from the command line type:
echo "[path]%~nx0" [output width] [cube faces fov] [Interpolator]
echo     [Input cube face image 1]...[cube face image 6]

if not "%EqInput%" == "" echo   [Equirectangular Input Image (optional)]

echo.
echo A maximum of 9 input arguments can be given to this program.
echo Default interpolator is %interp%, default cube face fov is 90.
echo.
echo The interpolator choices are as follows:
echo       poly3   spline16   spline36  spline64  sinc256  or  sinc1024
echo.

if not "%EqInput%" == ""  (
   echo If you specify an equirectangular input image as well, then it will be
   echo merged with any input cube faces and included in the output
   echo equirectangular image. 
   echo.
)

echo To run this program from Windows, just drag a cube face image(s) 
echo onto this program's icon or onto a shortcut pointing to this program.
echo (You can select MULTIPLE cube images simultaneously and drag them
echo onto the program icon.)
echo.
echo If you wish, you can create a shortcut pointing to this program,
echo and place the shortcut in your Windows 'Send To' folder. 
echo On Windows XP it would be located at   ~%USERNAME%\SendTo\ 
echo Then all you have to do is right-click on any cube image(s) you wish 
echo to convert, and choose 'SendTo' this program.
echo.
pause
goto done
REM *************************************************************************************************************
REM ************************************* Library of Subroutines *********************************************
REM *************************************************************************************************************
REM *************************************************************************************************************
REM *************************************************************************************************************
REM *************************************************************************************************************


REM Warnings about using 'if' statements with input arguments %1, %2, %3, etc...

REM Often times,  'if %var% =='   statement can give a program error because %var%
REM may contain spaces with multiple words. This confuses the if statement, unless
REM you are absolutely sure that %var% will not contains spaces.
REM You can get around this by using   if "%var%" ==
REM But, if var itself contains multiple words with spaces contained by quotes,
REM then     if "%var%" ==  will cause problems for you as well!
REM By convention, we will use  if "%var%"   in this program, only make sure that
REM our internal variables here do not contains multiple words with spaces contained by quotes.
REM And if you know that an internal variable is guaranteed to have quotes, then
REM simply use  if %var% ==    instead of   if "%var%" ==
REM Complicated, but that is DOS for you!

REM Note, however, that %1 will often already have string quotation marks in it.
REM I cannot use   if "%1" ==    statement because %1 can contain quotes itself
REM and this screws up the program sometimes, so we use another way.
REM Instead we can use   if "%~1" == 
REM Note that ~1 will remove any quotes (if any), and then "%~1" will guarantee
REM that only one set of quotes are present.

REM Another danger:
REM If %1 contains parentheses in it ()
REM and you use an 'if' statement that uses parentheses such as:
REM                if %1 == jjjhj (
REM                   command %1
REM                    )
REM 
REM We would get an error here as well! So, we need to be VERY careful with our if statement.
REM In the above case, just use quotes so any parentheses in %1 do not get mixed up with the
REM parentheses of the 'if' statement:
REM               if "%~1" == jjjhj (
REM                   command "%~1"
REM                      )


REM ***********************************************************************************************************
REM Print out the current version number of Dos for Panos
REM Usage:
REM       goto dfpversion
REM       :returndfpversion
REM
:dfpversion
echo DOS Utilities for Panos (DOSUP) v1.1b  By Eric Gerds  http://www.pinlady.net/vr
echo.
goto returndfpversion


REM *****************************************************************************************************
REM This is stuff that needs to be done at the very beginning of each Dos utility.
REM The variable 'searchreal' must be set=checknum or checkrealnum 
REM before running this subroutine.
:initialstuff

REM Default interpolation setting
REM The 'm' line in script sets the interpolation parameters
REM poly3 is worst, sinc1024 is best - except that sinc causes ringing at high contrast edges
REM i0=poly3, i1=spline16, i2=spline36, i4=spline64, i3=sinc256, i7=sinc1024
SET interpolate=i4
SET interp=spline64


REM Verify that command interpreter is cmd.exe and that OS is
REM Win 2000/XP 
goto verifyos
:returnverifyos


REM Verify that there are any input arguments at all to this program
REM If there were no input arguments, then give message and exit.
if "%~1" == "" goto instruct


REM Process command line '/?' argument
REM Display purpose of program and command line argument(s)
REM Check for /? BEFORE cls command because this means you are
REM already in DOS mode when you started this program.
REM We check ALL args for /? because a "call [subroutine] /?" later on would
REM mess up the program. I check each arg one by one just to be safe.
REM Do not use any 'for' loop or other fancy stuff to test for /? because it won't work.
if "%~1" == "/?" goto instruct
if "%~2" == "/?" goto instruct
if "%~3" == "/?" goto instruct
if "%~4" == "/?" goto instruct
if "%~5" == "/?" goto instruct
if "%~6" == "/?" goto instruct
if "%~7" == "/?" goto instruct
if "%~8" == "/?" goto instruct
if "%~9" == "/?" goto instruct


REM Check validity of PTStitcher
REM %PTStitch% is assumed to have quotes
goto getptstitcher
:returngetptstitcher


REM Search all input arguments for interpolation setting
call :getinterp %*


REM Now search for the first VALID image input argument.
REM Note that 'shift /1' will shift the input args %1 %2 %3 ..., but %* itself will NOT be shifted.
REM SET searchreal=checknum or checkrealnum   before calling  :searchfirstimage
call :searchfirstimage %*
:shifterloop1
if "%shiftargs%" neq "0" (
   shift /1
   SET /a shiftargs=shiftargs-1
   goto :shifterloop1
)

REM No input images found, so show instructions and exit program
if "%~1" == "" goto instruct

REM Valid Image now found

REM We set the default fov for all cube faces = 90 degrees
SET cubefov=90


REM Return from subroutine
goto :returninitialstuff


REM *****************************************************************************************************
REM Subroutine - Search the input line for the first input image found.
REM The output is 'shiftargs' which tells you by how many args to shift the input line.
REM Add this to the Library.
REM Input var is %searchreal% - which should = checkrealnum OR checknum
:searchfirstimage

SET shiftargs=0

:searchfirstimag1
REM If %1 argument is blank, then stop because we ran out of inputs to process
if "%~1" == "" goto :eof

if "%~1" == "/?" goto :searchfirstimag2

REM If we detect /r or /R then skip it, because we already scanned for this.
if /i "%~1" == "/R" goto :searchfirstimag2

REM If %1 is a real number, then skip it since we already scanned the input arguments
REM for yaw, pitch, roll.
REM We remove any quotes to checknum input using ~1
call :%searchreal% %~1
if "%validnum%" neq "" goto :searchfirstimag2

REM If %1 is interpolation method specification, then skip it, since
REM we already scanned the input arguments for interpolation method.
REM We remove any quotes to input for checkinterp
call :checkinterp %~1
if "%tmpinterpolate%" neq "" goto :searchfirstimag2

REM Check validity of Input Image %1
REM If Image file was specified but does not exist, then give error message
REM and then continue processing the next input argument.
if not exist "%~1" (
   call :mess3 "%~1"
   goto :searchfirstimag2
)

REM Verify the .tif .jpg .bmp extension for the input file %1
REM > nul will prevent the find command from outputting stuff to screen.
REM /I parameter means to ignore case.
echo "%~x1" | findstr /i ".tif .jpg .bmp" > nul
if %errorlevel% gtr 0 (
      call :notimage "%~1"
      goto :searchfirstimag2
)

REM Valid image found, so return from subroutine
goto :eof

:searchfirstimag2
SET /a shiftargs=shiftargs+1
shift /1
goto :searchfirstimag1


REM ***********************************************************************************************************
REM Check if PTStitcher is being registered. [This is when the user specifies the location of
REM PTStitcher on their hard drive]
REM If so, save it's path for later use with Dos for Panos.
REM The path will be saved in a DOS program pointed to by the variable 'dfpstitcher'
REM 'dfpstitcher' should point to a location that is always accessible, regardless of the current
REM working directory of Dos for Panos, or the value of %PATH%. 
REM %windir% is a good location, for example.
REM
REM If PTStitcher is not being registered, then find the path of PTStitcher.
REM If path not found, then give error message.
REM
REM Run this routine AFTER you are sure that cmd.exe is running and command extensions are on.
REM The return from this routine is :returngetptstitcher
REM 
REM To call this subroutine, type the following 2 lines
REM         goto getptstitcher
REM        :returngetptstitcher
:getptstitcher


REM Location of DOS helper that sets the value of 'PTStitch' variable,
REM which contains the PTStitcher.exe path.
SET dfpstitcher="%windir%\Dos4PanosStPath.bat"

REM PTStitcher will only be registered if it is given as the ONLY input argument

REM If no input args, then PTStitcher is NOT being registered.
REM So, see if PTStitcher exists and we can get it's path.
if "%~1" == "" goto getptstitcher1


SET tmp=

REM Determine if 'PTStitcher.exe' given as any one of the input file arguments.
for %%A in ("%~7" "%~6" "%~5" "%~4" "%~3" "%~2" "%~1") do (
    if /i "%%~nxA" == "PTStitcher.exe" SET tmp="%%~A"
)

REM If not, see if PTStitcher exists and we can get it's path.
REM %tmp% will contain quotes, so we cannot use quotes in this 'if' statement.
if a%tmp%a == aa goto :getptstitcher1


REM If we reach this point, then PTStitcher.exe is given as an input argument
REM with the intent that it be 'registered' with Dos For Panos.

REM Must use quotes such that PTStitch=" "
echo REM Set path of PanoTools PTStitcher.exe so it can work with Dos Utilities for Panos >  %dfpstitcher%
echo REM Dos Utilities For Panos  By Eric Gerds     http://www.pinlady.net/vr >>  %dfpstitcher%
echo. >>  %dfpstitcher%
echo SET PTStitch=%tmp% >>  %dfpstitcher%
echo. >>  %dfpstitcher%

cls
echo The path of PTStitcher is now registered as:
echo %tmp%
echo.
echo If this is the correct path, then Dos Utilities for Panos is ready to be used.
echo.
pause


REM PTStitcher has been registered, so now exit the program
goto :eof


REM Find PTStitcher.exe and get it's path. If not found, give error message.
:getptstitcher1

SET PTStitch=""

REM See if Dos program that sets PTStitcher path exists
if not exist %dfpstitcher% goto nostitcher

REM It exists, so run the file to set the stitcher path
call %dfpstitcher%

if %PTStitch% == "" goto nostitcher

REM PTStitcher found, so return from this subroutine
if exist %PTStitch% goto returngetptstitcher


REM Stitcher.exe not found, so give error message and then exit the program
:nostitcher
cls
echo Error! Cannot find PanoTools PTStitcher.exe
echo.
echo 'Dos Utilities For Panos' requires the correct path
echo to PTStitcher.exe in order to function properly.
echo. 
echo Please drag PTStitcher.exe onto any one of the Dos 
echo programs, and then try again.
echo.
echo.
echo.
pause 

REM Exit the program
goto :eof


REM *********************************************************************************************************
:error1
echo.
echo Error...
echo The resize scriptfile '%Script%' is missing.
echo Resizing of input image is aborted.
echo.
pause
goto :eof


REM ***************************************************************************************
REM SUBROUTINE - output error message
:mess3
echo.
echo Error...
echo Input argument does not appear to be valid.
echo Skipping argument: "%~1"
echo.
pause
echo.

REM Return from subroutine
goto :eof

REM *****************************************************************************************
REM SUBROUTINE
:notimage

REM %~nx1 means that for image %1, 
REM display only file name and extension without the path
echo.
echo Error...
echo File does not appear to have an acceptable  
echo extension such as .tif .jpg or .bmp
echo Skipping file: "%~nx1"
echo.
pause
echo.

REM Return from subroutine
goto :eof

REM ******************************************************************************************************
REM SUBROUTINE - Return only the first input argument, including any possible quotes
:getfirstarg
SET firstarg=
SET firstarg=%1
goto :eof

REM *******************************************************************************************************
REM SUBROUTINE  - Get keyboard input for positive integer. Verify all input characters.
REM This program is actually a subroutine that is called from another DOS batch routine.
REM INPUT is %1 argument, which is the input message displayed.
REM Output is %posnum%

:getnum
REM For some reason, the regular expressions in this DOS version do not work
REM completely correctly. For example, I cannot detect 5*5 as an invalid input
REM even though I would give the proper regular expression to detect this.
REM I believe that DOS is a little buggy in this regard. 

REM For some reason, I need to clear the variable using 'set posnum='
REM first before getting input with 'set /p posnum =', because if an invalid input is given the first time, 
REM that value remains in posnum if we execute 'set /p posnum =' a second time and just press the
REM carriage return as input.
set posnum=
set /p posnum=%~1

REM If posnum is completely empty, then exit this routine.
REM This is not so easy to check for, however, because posnum could have multiple words
REM separated by spaces, and may contain one or more sets of quotes.
REM 
REM If we just use    'if %posnum% =='  we could get an error when posnum contains more
REM than one word. Normally, using quotes would avoid that.
REM But, if we use  'if "%posnum%" =='  we could get an error if posnum contains quotes itself.
REM The most reliable method is to look at only the first word in posnum by calling :getfirstarg
REM and then testing that first word to see if it is empty or not.
call :getfirstarg %posnum%
if a%firstarg%a == aa goto finish

REM Note here that we do NOT use quotes around posnum when calling checknum.
REM The effect is that if posnum contains words separated by spaces, then each word is 
REM sent as a separate input argument to checknum. If checknum detects more than one 
REM input argument, then obviously posnum is not a valid single word number.
REM The only problem here comes up if posnum contains quotes, which I would like
REM to remove when sending it to checknum. We remove any quotes by using %posnum:"=%
REM So then if there are any leading/trailing spaces, they are removes when calling checknum,
REM and if there is more than one word in posnum, that will tell checknum that posnum is
REM not a valid 'single word' number.
REM The only hole here is if posnum contains one single " character, which will cause this
REM program to exit.
call :checknum %posnum:"=% 
if "%validnum%" == "" (
   echo wrong numerical input!
   echo The only permitted characters are 0 thru 9.  
   echo No commas or decimal points or other characters.
   echo.
   goto getnum
)

SET posnum=
SET posnum=%validnum%

REM Return from subroutine
:finish
set ttt=
goto :eof


REM *******************************************************************************************************
REM SUBROUTINE - Check if input parameter %1 is a positive number. Return integer value.
REM
REM Output is %validnum%, which will contain the INTEGER part of the positive number. 
REM So when you are done calling this routine, just use %validnum% as your number. 
REM If validnum is blank, then the input was not a positive number.
REM
REM It is better if the %1 has NO surrounding quotes in it, that way any leading/trailing spaces
REM on the input are automatically removed. 
REM To ensure this, you can call this routine using:
REM call :checknum %var:"=%      or
REM call :checknum %~1              if using an input argument from a routine.             
REM
REM This way, if the input to this routine has more than one word in it, the words are assigned 
REM to %2, %3 etc..., which means that the input variable to this routine had more than
REM one word, and so is not a valid single number.
REM Check if a %2 exists. If so, then there was more than one word as input.
REM
REM BUG NOTE:  findstr /R "[^0-9]" does not work correctly for some reason, but 
REM findstr /R "^[^0-9]" and findstr /R "$[^0-9]"    do work.
 
:checknum

REM echo.
REM echo %1

set validnum=
REM We use quotes around %1, just in case we forget to remove them when calling this routine.
REM This avoids an error in the 'if' statement.
REM Note: We cannot say   'if %~1 ==' 
REM since %1 may be a string with multiple words, which gives an error.
if "%~1" == "" goto :eof

REM If %2 is not blank, then we have a multiword input, so invalid number
if "%~2" neq "" goto :eof

REM Make sure there are numerical digits in %1
REM We use regular expression here.
REM Note, I use %1 and not %~1 because it is ok to keep the
REM surrounding quotes that %1 may have in the next regular expression.
echo %~1 | findstr /i "0 1 2 3 4 5 6 7 8 9" > nul
if %errorlevel% gtr 0 goto :eof

REM We remove all numeric digits from the number
REM If anything is left over, then we have a non number.
REM We have to remove spaces as well.
SET t1=%~1
if "%t1%" neq "" SET t1=%t1:0=%
if "%t1%" neq "" SET t1=%t1:1=%
if "%t1%" neq "" SET t1=%t1:2=%
if "%t1%" neq "" SET t1=%t1:3=%
if "%t1%" neq "" SET t1=%t1:4=%
if "%t1%" neq "" SET t1=%t1:5=%
if "%t1%" neq "" SET t1=%t1:6=%
if "%t1%" neq "" SET t1=%t1:7=%
if "%t1%" neq "" SET t1=%t1:8=%
if "%t1%" neq "" SET t1=%t1:9=%
if "%t1%" neq "" SET t1=%t1:.=%
if "%t1%" neq "" SET t1=%t1:+=%
if "%t1%" neq "" SET t1=%t1: =%

REM If anything left over, even a minus sign, then we do not have a positive number
if "%t1%" neq "" goto :eof

REM Our routine does not properly deal with -0 or -00 etc...
REM But this is ok for now

REM More than one '.' in string is error
echo %~1 | findstr /R "\..*\." > nul
if %errorlevel% equ 0 goto :eof

REM + sign can ONLY be at beginning of string
echo %~1 | findstr /R ".+" > nul
if %errorlevel% equ 0 goto :eof


SET t1=
SET t1=%~1

REM Remove + sign, not needed
if "%t1%" neq "" SET t1=%t1:+=%


REM Remove leading zeros from number
REM Leading zeros interpreted as HEX or OCTAL BASE?
:checknum1
REM /R "^0." does not seem to work correctly, so we use /R "^0[0-9]"
echo %t1% | findstr /R "^0[0-9]" > nul
if %errorlevel% gtr 0 goto :checknum2
SET t1=abc%t1%
SET t1=%t1:abc0=%
goto :checknum1
:checknum2


REM If number=.XXXX we convert to number=0.XXXXX
REM We do this conversion BEFORE taking integer value of number.
echo %t1% | findstr /R "^\." > nul
if %errorlevel% equ 0 SET t1=0%t1%

REM Is there a way to take the integer value of a decimal number? Yes.
REM Replace decimal point with space, then call subroutine that
REM returns the first argument.
REM Replace decimal point (.) with space
if "%t1%" neq "" SET t1=%t1:.= %
call :getfirstarg1 %t1%


REM Return from subroutine
goto :eof

REM Return first argument without any quotes
:getfirstarg1
SET validnum=%~1
goto :eof



REM *******************************************************************************************************
REM SUBROUTINE - Check if input parameter %1 is a number (positive, negative, 
REM integer, non integer, zero). Decimal numbers are assumed to have '.' and NOT ','
REM
REM Output is that real number in the variable %validnum%.  
REM So when you are done calling this routine, just use %validnum% as your number. 
REM If validnum is blank, then the input was not a valid number.
REM
REM It is better if the %1 has NO surrounding quotes in it, that way any leading/trailing spaces
REM on the input are automatically removed. 
REM To ensure this, you can call this routine using:
REM call :checkrealnum %var:"=%      or
REM call :checkrealnum %~1              if using an input argument from a routine.             
REM
REM This way, if the input to this routine has more than one word in it, the words are assigned 
REM to %2, %3 etc..., which means that the input variable to this routine had more than
REM one word, and so is not a valid single number.
REM Check if a %2 exists. If so, then there was more than one word as input.
REM
REM BUG NOTE:  findstr /R "[^0-9]" does not work correctly for some reason, but 
REM findstr /R "^[^0-9]" and findstr /R "$[^0-9]"    do work.
 
:checkrealnum

REM echo.
REM echo %1

set validnum=
REM We use quotes around %1, just in case we forget to remove them when calling this routine.
REM This avoids an error in the 'if' statement.
REM Note: We cannot say   'if %~1 ==' 
REM since %1 may be a string with multiple words, which gives an error.
if "%~1" == "" goto :eof

REM If %2 is not blank, then we have a multiword input, so invalid number
if "%~2" neq "" goto :eof

REM Make sure there are numerical digits in %1
REM We use regular expression here.
REM Note, I use %1 and not %~1 because it is ok to keep the
REM surrounding quotes that %1 may have in the next regular expression.
echo %~1 | findstr /i "0 1 2 3 4 5 6 7 8 9" > nul
if %errorlevel% gtr 0 goto :eof

REM We remove all numeric digits from the number
REM If anything is left over, then we have a non number.
REM We have to remove spaces as well.
SET t1=%~1
if "%t1%" neq "" SET t1=%t1:0=%
if "%t1%" neq "" SET t1=%t1:1=%
if "%t1%" neq "" SET t1=%t1:2=%
if "%t1%" neq "" SET t1=%t1:3=%
if "%t1%" neq "" SET t1=%t1:4=%
if "%t1%" neq "" SET t1=%t1:5=%
if "%t1%" neq "" SET t1=%t1:6=%
if "%t1%" neq "" SET t1=%t1:7=%
if "%t1%" neq "" SET t1=%t1:8=%
if "%t1%" neq "" SET t1=%t1:9=%
if "%t1%" neq "" SET t1=%t1:.=%
if "%t1%" neq "" SET t1=%t1:+=%
if "%t1%" neq "" SET t1=%t1:-=%
if "%t1%" neq "" SET t1=%t1: =%

if "%t1%" neq "" goto :eof


REM More than one '.' in string is error
echo %~1 | findstr /R "\..*\." > nul
if %errorlevel% equ 0 goto :eof

REM + sign can ONLY be at beginning of string
echo %~1 | findstr /R ".+" > nul
if %errorlevel% equ 0 goto :eof

REM - sign can ONLY be at beginning of string
echo %~1 | findstr /R ".\-" > nul
if %errorlevel% equ 0 goto :eof

REM See if any non zero numerical digits present (1-9)
REM If none present, but we know that numerical digits ARE present,
REM then they must be all zeros.
echo %~1 | findstr /R "[1-9]" > nul
if %errorlevel% gtr 0 (
     SET validnum=0
     goto :eof  
)


SET t1=
SET t1=%~1


REM AFTER checking for + at start of string, remove + sign (not needed)
if "%t1%" neq "" SET t1=%t1:+=%

REM Remove leading zeros from number
REM Leading zeros interpreted as HEX or OCTAL BASE?
:checkrealnum1
REM /R "^0." does not seem to work correctly, so we use /R "^0[0-9]"
echo %t1% | findstr /R "^0[0-9]" > nul
if %errorlevel% gtr 0 goto :checkrealnum2
SET t1=abc%t1%
SET t1=%t1:abc0=%
goto :checkrealnum1
:checkrealnum2


REM Remove trailing zeros after decimal point
REM If number=XXXX.Y000 we convert to number=XXXX.Y 
:checkrealnum3
SET t1=%t1%abc
echo %t1% | findstr /R "\.[0-9][0-9]*0abc" > nul
if %errorlevel% gtr 0 goto :checkrealnum4 
    SET t1=%t1:0abc=%
    goto :checkrealnum3
:checkrealnum4

REM At this point, t1 still ends with 'abc', but that is ok because we use this below.

REM If number=XXXX. we convert to number=XXXXX.0
REM Bug in DOS prevents findstr /R "[0-9]*\.$" from working.
REM So we try this trick below.
SET t1=%t1:.abc=.0%
SET t1=%t1:abc=%


SET validnum=%t1%
REM Return from subroutine
goto :eof


REM *******************************************************************************************************
REM SUBROUTINE - Check for width and cube face fov specification in the input arguments.
REM Input is ALL the input arguments to this program %1, %2, ... %N
REM If valid numbers found, then the 'width' variable and 'cubefov' variables will be set.

:getpanowidth
if "%~1" == "" goto :eof

REM Get width, must be > 0, integer
REM We use ~1 to remove any possible quotes.
call :checknum %~1
shift /1
if "%validnum%" == ""  goto :getpanowidth

SET width=
SET width=%validnum%

:getpanowidth1
if "%~1" == "" goto :eof

REM Get cube face fov.
REM fov must be real number, >0 and < 180.
REM But since 'if' does not work with decimal point, we call :checknum instead of 
REM checkrealnum to make sure we have a number, and get back a positive integer
call :checknum %~1
if "%validnum%" == "" (
    shift /1
    goto :getpanowidth1
)

REM Does 'if' work on numbers with a decimal point? No.
REM So we had checknum return the integer part of the number.
if %validnum% gtr 179 goto :eof
if %validnum% lss 0 goto :eof

REM Now we call checkrealnum, which will format the real number for us.
call :checkrealnum %~1
if "%validnum%" == "" goto :eof

SET cubefov=%validnum%

REM Return from subroutine
goto :eof


REM *******************************************************************************************************
REM SUBROUTINE - get interpolation parameter anywhere in the input line.
REM Input is ALL the input arguments to this program %1, %2, ... %N

:getinterp
if "%~1" == "" goto :eof

REM Check %1, if it is interpolator param, then set variables and exit routine
REM We remove any quotes to input for checkinterp
call :checkinterp %~1
if "%tmpinterpolate%" neq "" (
   SET interpolate=%tmpinterpolate%
   SET interp=%tmpinterp%
   goto :eof
)

REM Else shift the paramaters to check the next input argument
shift /1
goto getinterp

REM Return from subroutine
goto :eof

REM *******************************************************************************************************
REM SUBROUTINE - Check for interpolator specification in the input argument %1
REM Input is %1, output is %tmpinterpolate%, %tmpinterp%
REM if the %1 input is poly3, spline16, etc... then set the output variables accordingly.

:checkinterp
set tmpinterpolate=
set tmpinterp=

REM If more than one input argument, then error
if "%~2" neq "" goto :eof

REM Check for interpolation parameter, case insensitive search.
REM i0=poly3, i1=spline16, i2=spline36, i4=spline64, i3=sinc256, i7=sinc1024
if /i "%~1" == "poly3" (
   SET tmpinterpolate=i0
   SET tmpinterp=poly3
   goto :eof
 )
if /i "%~1" == "spline16" (
   SET tmpinterpolate=i1
   SET tmpinterp=spline16
   goto :eof
 )
if /i "%~1" == "spline36" (
   SET tmpinterpolate=i2
   SET tmpinterp=spline36
   goto :eof
 )
if /i "%~1" == "spline64" (
   SET tmpinterpolate=i4
   SET tmpinterp=spline64
   goto :eof
 )
if /i "%~1" == "sinc256" (
   SET tmpinterpolate=i3
   SET tmpinterp=sinc256
   goto :eof
 )
if /i "%~1" == "sinc1024" (
   SET tmpinterpolate=i7
   SET tmpinterp=sinc1024
   goto :eof
 )


REM Return from subroutine
goto :eof

REM ****************************************************************************
REM Verify that we are running cmd.exe and NOT command.com.
REM We need the extra commands found in cmd.exe.
REM Also verify that Win 2000/XP are running, which have the
REM cmd.exe command interpreter.
REM Note, this subroutine cannot be 'called', you have to use 'goto'
REM because we do not know yet whether the command interpreter
REM can handle it.
REM Also, we cannot use equ, neq, gtr etc.. with 'if' statement because
REM we do not know yet if command extensions are active.
:verifyos

REM This program uses some of cmd.exe's new commands such
REM as 'set /p' and others.
REM Also verify that cmd.exe command extensions are ON

REM Note that Command.com does not have a findstr command,
REM so we must use find instead here until we know that
REM cmd.exe is running.
echo %comspec% | find /i "cmd.exe" > nul

REM Note that if we are running COMMAND.COM, we cannot use
REM '%errorlevel% gtr' command here!
if errorlevel == 2 goto wrongcom
if errorlevel == 1 goto wrongcom
REM if errorlevel == 0 then cmd.exe is running


REM Test to verify that DOS command extensions are active
REM so that we can use them in this program.
REM The 'set /a' is a command extension, if it works, then extensions
REM are active. Do not use 'if xxx neq' here.
set t1=10
set t2=
set /a t2=%t1%/2
if not a%t2%a == a5a goto extoff


REM Check OS version. Must be 2000 or XP. Why?
REM Because the SET /P command (which we use later to get user input) is only on 2000 / XP
REM 

REM Check for Windows XP
REM /C means to use search string literally - which means that 
REM the space in " XP" is included in the search. 
REM echo %vers% | findstr /i /C:" XP" > nul
ver | findstr /i /C:" XP" > nul
if %errorlevel% equ 0 goto osok

REM Check for Windows 2000
REM echo %vers% | findstr /i /C:" 2000" > nul
ver | findstr /i /C:" 2000" > nul
if %errorlevel% equ 0 goto osok

goto wrongos

:osok

REM WINDOWS XP/2000 and cmd.exe detected, so continue.
goto returnverifyos

REM ****************************************************************
:wrongos
echo Error! This DOS program will only run under Win XP or Win 2000.
echo.
pause
REM We cannot use goto :eof because of wrong OS
goto done

REM ******************************************************************
:wrongcom
echo cmd.exe is currently not running, but is required
echo for this program. 
echo The cmd.exe DOS shell runs under Windows 2000/XP.
echo Program aborted.
echo.
pause
REM We cannot use goto :eof because of wrong OS
goto done

REM *******************************************************************
:extoff
echo cmd.exe DOS interpreter command extensions are off.
echo Make sure you are running Windows 2000/XP and
echo that your DOS command interpreter is cmd.exe with extensions ON.
echo Program aborted.
echo.
pause
goto done

REM *******************************************************************************************************
REM We do not use the 'exit' command so as not to close the DOS window when the
REM user manually opened a DOS window in the first place.
:done


